﻿using DotNetAuth.Profiles;
using System;
using Wisej.Web;

namespace Wisej.OpenAuth
{
	public class AuthenticationManager : ILoginStateManager
	{
		private string stateCode;
		private Action<Profile> callback;
		private LoginProvider provider;

		public AuthenticationManager(LoginProviderDefinition provider, string appId, string appSecret)
		{
			this.RequiredProperties = new[] {
				ProfileProperty.Email
			};

			LoginProvider.SetKeys(provider, appId, appSecret);
			this.provider = LoginProvider.Get(provider.Name);
		}

		/// <summary>
		/// Profile properties.
		/// </summary>
		public ProfileProperty[] RequiredProperties
		{
			get;
			set;
		}

		#region ILoginStateManager

		void ILoginStateManager.ClearTemp()
		{
			this.stateCode = null;
		}

		string ILoginStateManager.LoadTemp()
		{
			return this.stateCode;
		}

		void ILoginStateManager.SaveTemp(string stateCode)
		{
			this.stateCode = stateCode;
		}

		#endregion

		public void Authenticate(Action<Profile> callback)
		{
			this.callback = callback;

			var authUrl = Login.GetAuthenticationUri(
				this.provider,
				Application.StartupUri,
				this,
				this.RequiredProperties);
			
			authUrl.Wait();
			Application.Navigate(authUrl.Result.ToString());
			Application.ApplicationRefresh -= this.AuthenticationResponse;
			Application.ApplicationRefresh += this.AuthenticationResponse;
		}

		private void AuthenticationResponse(object sender, EventArgs e)
		{
			Application.ApplicationRefresh -= this.AuthenticationResponse;

			var profile = DotNetAuth.Profiles.Login.GetProfile(
				provider,
				Application.Uri,
				Application.StartupUrl,
				this, 
				this.RequiredProperties);

			profile.Wait();
			callback(profile.Result);

			// clean up the URL.
			Application.Navigate(Application.StartupUrl);
		}
	}
}